#!/usr/bin/env python
# -*- coding: utf-8 -*-
from string import ascii_letters
from random import choice

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import TestCase, override_settings

from ..models import Entry


@override_settings(SITE_ID=1)
@override_settings(ROOT_URLCONF='blargg.tests.urls')
class TestEntrySitemap(TestCase):

    def setUp(self):
        username = ''.join([choice(ascii_letters) for i in range(10)])
        User = get_user_model()
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )
        self.entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            tag_string="foo, bar, baz",
        )
        self.entry.publish()  # Calls save

    def test_sitemap_root(self):
        # The root sitemap doc should include a link to the blog
        resp = self.client.get('/sitemap.xml')
        self.assertEqual(resp.status_code, 200)
        self.assertContains(resp, 'sitemap-blog.xml')

    def test_sitemap_blog(self):
        # The sitemap for blogs should contain a link to an Entry
        resp = self.client.get('/sitemap-blog.xml')
        self.assertEqual(resp.status_code, 200)
        self.assertContains(resp, self.entry.get_absolute_url())



from django import template
from django.urls import reverse
from blargg.models import Entry

register = template.Library()


@register.simple_tag
def entry_archive_year_url():
    """Renders the ``entry_archive_year`` URL for the latest ``Entry``."""
    entry = Entry.objects.filter(published=True).latest()
    arg_list = [entry.published_on.strftime("%Y")]
    return reverse('blargg:entry_archive_year', args=arg_list)



"""
This module defines a default Sitemap for ``Entry``'s. To enable sitemaps,
you'll need to configure your project's URL patters with something like:

    from blargg.sitemaps import EntrySitemap

    sitemaps = {'blog': EntrySitemap}

    urlpatterns = patterns('',

        url(
            r'^sitemap\.xml$',
            'django.contrib.sitemaps.views.sitemap',
            {'sitemaps': sitemaps}
        )
    )

"""
from django.contrib.sitemaps import Sitemap
from .models import Entry


class EntrySitemap(Sitemap):
    changefreq = "never"
    priority = 0.5

    def items(self):
        return Entry.objects.filter(published=True)

    def lastmod(self, obj):
        return obj.updated_on



from django.contrib import admin
from . import models


class TagAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug')
    search_fields = ('name', )


class EntryAdmin(admin.ModelAdmin):
    list_display = (
        'title', 'content_format', 'author', 'published',
        'published_on', 'updated_on'
    )
    date_hierarchy = 'created_on'
    list_filter = ('published', 'content_format')
    search_fields = ('title', 'raw_content', 'tag_string')
    prepopulated_fields = {"slug": ("title", )}
    actions = ['publish_entries']

    def publish_entries(self, request, queryset):
        for entry in queryset:
            entry.publish()

admin.site.register(models.Tag, TagAdmin)
admin.site.register(models.Entry, EntryAdmin)



from django.conf.urls import url, include
from django.contrib.sitemaps import views as sitemaps_views
from blargg.sitemaps import EntrySitemap


sitemaps = {
    'blog': EntrySitemap,
}


urlpatterns = [
    url(r'^blog/', include('blargg.urls', namespace='blargg')),

    # Sitemaps
    url(
        r'^sitemap\.xml$',
        sitemaps_views.index,
        {'sitemaps': sitemaps, 'sitemap_url_name': 'sitemaps'}
    ),
    url(
        r'^sitemap-(?P<section>.+)\.xml$',
        sitemaps_views.sitemap,
        {'sitemaps': sitemaps},
        name='sitemaps'
    ),
]



# Generated by Django 2.1.4 on 2018-12-28 18:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blargg', '0002_auto_20181228_1711'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entry',
            name='content_format',
            field=models.CharField(choices=[('md', 'Markdown'), ('rst', 'reStructured Text'), ('html', 'HTML')], max_length=4),
        ),
    ]



"""
Custom settings for django-blargg.

* ``mail2blogger`` -- enable Mail2Blogger. Published entries will be
  cross-posted to Blogger.
* ``mail2blogger_email`` -- the email address to which published entries are
  mailed.

"""

BLARGG = {
    'title': 'A Blargg Blog',
    'description': 'Blog Entries',
    'mail2blogger': False,
    'mail2blogger_email': '',
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from string import ascii_letters
from random import choice

try:
    from unittest.mock import Mock, patch
except ImportError:
    from mock import Mock, patch

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import TestCase, override_settings
from django.utils.timezone import now as utc_now

from ..models import Tag, Entry


@override_settings(SITE_ID=1)
@override_settings(ROOT_URLCONF='blargg.tests.urls')
class TestTagManager(TestCase):

    def setUp(self):
        username = ''.join([choice(ascii_letters) for i in range(10)])
        User = get_user_model()
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )
        self.entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            tag_string="foo, bar, baz",
        )

    def test_create_tags(self):
        """This kinda sucks, but saving an entry triggers a signal that then
        creates tags via TagManager

        Entry.save()
            -> generate_entry_tags()
                -> Tag.objects.create_tags(entry)

        """
        # No Tags yet...
        self.assertEqual(Tag.objects.all().count(), 0)
        # Save the Entry
        self.entry.save()
        # We should have tags!
        self.assertEqual(Tag.objects.all().count(), 3)


@override_settings(ROOT_URLCONF='blargg.tests.urls')
class TestTag(TestCase):

    def setUp(self):
        self.tag = Tag(name="Test Tag")
        self.tag.save()

    def test__str__(self):
        self.assertEqual(self.tag.__str__(), u"test tag")

    def test_save(self):
        """Verify that tag names get lowercased & stripped, while slugs are
        slugified."""
        t = Tag(name="   Foo Tag   ")
        self.assertEqual(t.slug, '')
        t.save()
        self.assertEqual(t.name, u"foo tag")
        self.assertEqual(t.slug, u"foo-tag")

    def test_get_absolute_url(self):
        self.assertIn("tags/test-tag/", self.tag.get_absolute_url())


@override_settings(SITE_ID=1)
@override_settings(ROOT_URLCONF='blargg.tests.urls')
class TestEntry(TestCase):

    def setUp(self):
        # patch the utc_now function so it's return value is essentiall cashed
        self.now = utc_now()
        config = {'return_value': self.now}
        self.utc_now_patcher = patch('blargg.models.utc_now', **config)
        self.utc_now_patcher.start()

        username = ''.join([choice(ascii_letters) for i in range(10)])
        User = get_user_model()
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )
        self.entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            tag_string="foo, bar, baz",
        )
        self.entry.save()

    def tearDown(self):
        self.utc_now_patcher.stop()

    def test__str__(self):
        self.assertEqual(self.entry.__str__(), self.entry.title)

    def test__create_slug(self):
        e = Entry(title="Foo Bar")
        e._create_slug()
        self.assertEqual(e.slug, "foo-bar")

    def test__create_date_slug_new(self):
        # The expected ``date_slug`` value
        date_slug = "{0}/{1}".format(self.now.strftime("%Y/%m/%d"), "test")

        # When entry is a new object (calls utc_now)
        e = Entry(title="test", slug="test")
        e._create_date_slug()
        self.assertEqual(e.date_slug, date_slug)

    def test__create_date_slug_published(self):
        # The expected ``date_slug`` value
        date_slug = "{0}/{1}".format(self.now.strftime("%Y/%m/%d"),
            self.entry.slug)

        # When entry is published (use the published_on date)
        self.entry.published = True
        self.entry.published_on = self.now
        self.entry._create_date_slug()
        self.assertEqual(self.entry.date_slug, date_slug)

    def test__create_date_slug_unpublished(self):
        # When entry is not published (use the updated_on date)
        self.entry.published = False
        self.entry.save()  # save to get an updated_on
        date_slug = "{0}/{1}".format(
            self.entry.updated_on.strftime("%Y/%m/%d"),
            self.entry.slug
        )
        self.entry._create_date_slug()
        self.assertEqual(self.entry.date_slug, date_slug)

    def test__render_content_html(self):
        self.entry.content_format = "html"
        self.entry._render_content()
        self.assertEqual(self.entry.raw_content, self.entry.rendered_content)

    def test__render_content_rst(self):
        self.entry.content_format = "rst"
        self.entry._render_content()
        results = self.entry.rendered_content.strip()
        self.assertEqual(results, "<p>Test Content</p>")

    def test__render_content_md(self):
        self.entry.content_format = "md"
        self.entry._render_content()
        results = self.entry.rendered_content.strip()
        self.assertEqual(results, "<p>Test Content</p>")

    def test__set_published(self):
        self.assertFalse(self.entry.published)
        self.assertEqual(self.entry.published_on, None)
        self.entry._set_published()
        self.assertTrue(self.entry.published)
        self.assertEqual(self.entry.published_on, self.now)

    def test_save(self):
        self.entry._create_slug = Mock()
        self.entry._create_date_slug = Mock()
        self.entry._render_content = Mock()
        self.entry._set_published = Mock()

        self.entry.save()

        self.entry._create_slug.assert_called_once_with()
        self.entry._create_date_slug.assert_called_once_with()
        self.entry._render_content.assert_called_once_with()
        # Entry was not published, so ``_set_published`` should not get called
        self.assertFalse(self.entry._set_published.called)

        self.entry.published = True
        self.entry.save()
        self.entry._set_published.assert_called_once_with()

    def test_get_absolute_url(self):
        expected = '/blog/{0}/'.format(self.entry.slug)
        self.assertEqual(self.entry.get_absolute_url(), expected)

    def test_get_absolute_url_with_date(self):
        # If not published /blog/slug/
        expected = '/blog/{0}/'.format(self.entry.slug)
        self.assertEqual(self.entry.get_absolute_url_with_date(), expected)

        # If published, /blog/yyyy/mm/dd/slug/
        self.entry.publish()
        expected= "/blog/{0}/{1}/".format(self.now.strftime("%Y/%m/%d"), self.entry.slug)
        self.assertEqual(self.entry.get_absolute_url_with_date(), expected)

    @patch.object(Entry, "_set_published")
    def test_publish(self, mock_set_published):
        self.entry.publish()
        mock_set_published.assert_called_once_with()

    @patch.object(Entry, "save")
    def test_unpublish(self, mock_save):
        self.entry.unpublish()
        self.assertFalse(self.entry.published)
        self.assertEqual(self.entry.published_on, None)
        mock_save.assert_called_once_with()

    def test_content(self):
        with patch("blargg.models.mark_safe") as mock_mark_safe:
            mock_mark_safe.return_value = "SAMPLE CONTENT"
            result = self.entry.content
            mock_mark_safe.assert_called_once_with(self.entry.rendered_content)
            self.assertEqual(result, "SAMPLE CONTENT")
        mock_mark_safe.reset_mock()

    def test_crossposted_content(self):
        with patch("blargg.models.mark_safe") as mock_mark_safe:
            mock_mark_safe.return_value = "SAMPLE CONTENT"
            result = self.entry.crossposted_content
            mock_mark_safe.assert_any_call
            self.assertEqual(result, "SAMPLE CONTENT")
        mock_mark_safe.reset_mock()



from django.conf import settings
from django.contrib.admin.sites import AdminSite
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import TestCase, override_settings

from ..models import Entry
from ..admin import TagAdmin, EntryAdmin


@override_settings(SITE_ID=1)
class TestTagAdmin(TestCase):
    """Verify expected fields on ``TagAdmin`` class"""
    def test_list_display(self):
        self.assertEqual(TagAdmin.list_display, ('name', 'slug'))

    def test_search_fields(self):
        self.assertEqual(TagAdmin.search_fields, ('name', ))


@override_settings(SITE_ID=1)
class TestEntryAdmin(TestCase):
    """Verify fields and methods on ``EntryAdmin`` class."""

    def test_list_display(self):
        expected = sorted([
            'title',
            'content_format',
            'author',
            'published',
            'published_on',
            'updated_on'
        ])
        self.assertEqual(sorted(EntryAdmin.list_display), expected)

    def test_date_hierarchy(self):
        self.assertEqual(EntryAdmin.date_hierarchy, 'created_on')

    def test_list_filter(self):
        self.assertEqual(EntryAdmin.list_filter, ('published', 'content_format'))

    def test_search_fields(self):
        expected = ('title', 'raw_content', 'tag_string')
        self.assertEqual(EntryAdmin.search_fields, expected)

    def test_prepopulated_fields(self):
        self.assertEqual(EntryAdmin.prepopulated_fields, {"slug": ("title", )})

    def test_actions(self):
        self.assertEqual(EntryAdmin.actions, ['publish_entries'])

    def test_publish_entries(self):
        # Some setup (need a User and an Entry)
        User = get_user_model()
        username = 'entryadmin_publish_entries'
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )
        entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
        )
        entry.save()
        self.assertFalse(entry.published)

        # Create an EntryAdmin and call the `publish_entries` method
        admin = EntryAdmin(Entry, AdminSite())
        admin.publish_entries(None, Entry.objects.all())

        # Fetch the Entry, and see if it's published.
        entry = Entry.objects.get(pk=entry.id)
        self.assertTrue(entry.published)



from django.urls import path
from django.views.generic import ArchiveIndexView
from django.views.generic import ListView

from .models import Entry, Tag
from .views import (
    EntryDayArchiveView,
    EntryDetailView,
    EntryMonthArchiveView,
    EntryYearArchiveView,
    TaggedEntryListView,
)

# URL examples
# ------------
# /blog/tags/                   -- tag list detail
# /blog/tags/foo/               -- entries tagged with "foo"
# /blog/a-sample-entry/         -- entry detail
# /blog/2013/01/05/             -- entry list (by year, month, day)
# /blog/2013/01/                -- entry list (by year, month)
# /blog/2013/                   -- entry list (by year)
# /blog/2013/05/a-sample-entry/ -- entry detail (with date slug)
# /blog/                        -- entry detail (latest published post)

app_name = 'blargg'
urlpatterns = [
    path('tags/', ListView.as_view(model=Tag), name='list_tags'),
    path(
        'tags/<slug:tag_slug>/',
        TaggedEntryListView.as_view(),
        name='tagged_entry_list'
    ),

    # Year, Month, Day Archives
    path(
        '<int:year>/<int:month>/<int:day>/',
        EntryDayArchiveView.as_view(),
        name='entry_archive_day'
    ),
    path(
        '<int:year>/<int:month>/',
        EntryMonthArchiveView.as_view(),
        name='entry_archive_month'
    ),
    path(
        '<int:year>/',
        EntryYearArchiveView.as_view(),
        name='entry_archive_year'
    ),

    # Detail views (with and without the date)
    path(
        '<int:year>/<int:month>/<int:day>/<slug:slug>/',
        EntryDetailView.as_view(),
        name='entry_detail'
    ),
    path('<slug:slug>/', EntryDetailView.as_view(), name='entry_detail'),
    path(
        '',
        ArchiveIndexView.as_view(
            model=Entry,
            date_field='published_on',
            paginate_by=10
        ),
        name='list_entries'
    ),
]



from setuptools import setup
from blargg import __version__

setup(
    name='django-blargg',
    version=__version__,
    description="A minimal, admin-powered, django blogging app with a fun name.",
    long_description=open('README.md').read(),
    author='Brad Montgomery',
    author_email='brad@bradmontgomery.net',
    url='https://github.com/bradmontgomery/django-blargg',
    license='MIT',
    packages=['blargg'],
    include_package_data=True,
    package_data={
        '': ['README.md', 'LICENSE.txt', 'HISTORY.md'],
        'blargg': ['templates/blargg/*.html']
    },
    zip_safe=False,
    install_requires=['django'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.dispatch import Signal, receiver
from django.template.defaultfilters import striptags

try:
    blargg_settings = settings.BLARGG
except AttributeError:  # pragma: no cover
    # Use the local settings
    from .settings import BLARGG  # pragma: no cover
    blargg_settings = BLARGG  # pragma: no cover

# -------------------
# Signal Definitions
# -------------------

entry_published = Signal(providing_args=["entry"])


# -------------------
# Signal Handlers
# -------------------

@receiver(entry_published, dispatch_uid='blargg-mail2blogger')
def mail2blogger(entry, **kwargs):
    """This signal handler cross-posts published ``Entry``'s to Blogger. For
    this to work, the following settings must be non-False; e.g.:

        BLARGG = {
            'mail2blogger': True,
            'mail2blogger_email': 'user@example.com',
        }

    """
    enabled = blargg_settings.get('mail2blogger', False)
    recipient = blargg_settings.get('mail2blogger_email', None)
    if enabled and recipient:

        # Send HTML (and text-only) email
        msg = EmailMultiAlternatives(
            entry.title,  # Subject
            striptags(entry.crossposted_content),  # Text-only
            settings.DEFAULT_FROM_EMAIL,  # From
            [recipient]  # List of Recipients
        )
        msg.attach_alternative(entry.crossposted_content, "text/html")
        msg.send(fail_silently=True)



# Generated by Django 2.1.4 on 2018-12-28 17:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('blargg', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entry',
            name='author',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='entry',
            name='published',
            field=models.BooleanField(blank=True, default=False, help_text='Show this entry to the public'),
        ),
    ]
